﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.IO;
using System.Drawing;

namespace RoboRace
{
    public class Board
    {
        public string BaseUri = "http://localhost:8000/";

        public char[,] Policka;
        public char[] Moznosti;
        public char[] Vyber;
        public int ID;
        public int X, Y;
        public char Smer;

        public Board()
        {
            Policka = new char[20, 20];
            Moznosti = new char[9];
        }

        // zahájí hru
        public void Start()
        {
            StreamReader reader = new StreamReader(WebRequest.Create(BaseUri + "start").GetResponse().GetResponseStream());
            LoadData(reader);
        }

        // vykoná vybrané kroky
        public void Execute()
        {
            string uri = BaseUri + "kolo?id=" + Convert.ToString(ID) + "&pohyby=" + (new string(Vyber));
            StreamReader reader = new StreamReader(WebRequest.Create(uri).GetResponse().GetResponseStream());
            LoadData(reader);
        }

        // načte plochu ze StreamReaderu
        private void LoadData(StreamReader reader)
        {
            string line;
            string[] parts;
            int y, x, i;

            line = reader.ReadLine();

            if (line.StartsWith("Vyhrali jste za ") || line.StartsWith("Chyba: "))
            {
                throw new InvalidOperationException(line);
            }

            ID = Convert.ToInt32(line);
            for (y = 0; y < 20; y++)
            {
                line = reader.ReadLine();
                for (x = 0; x < 20; x++)
                {
                    Policka[y, x] = line[x];
                }
            }

            parts = reader.ReadLine().Split(' ');
            X = Convert.ToInt32(parts[0]) - 1;
            Y = Convert.ToInt32(parts[1]) - 1;
            Smer = parts[2][0];

            line = reader.ReadLine();
            for (i = 0; i < 9; i++)
            {
                Moznosti[i] = line[i];
            }

            reader.Close();
        }

        public List<dynamic> CalculateMoves(out bool win, out bool gameOver)
        {
            return CalculateMoves(Vyber, X, Y, Smer, out win, out gameOver);
        }

        // spočítá cestu, po které robot půjde
        public List<dynamic> CalculateMoves(char[] moves, int x, int y, char smer, out bool win, out bool gameOver)
        {
            List<dynamic> kroky = new List<dynamic>();

            int deltaX = 0, deltaY = 0;
            int i;

            switch (smer)
            {
                case 'N': deltaY = -1; break;
                case 'R': deltaX = 1; break;
                case 'L': deltaX = -1; break;
                case 'D': deltaY = 1; break;
            }
            foreach (char move in moves)
            {
                switch (move)
                {
                    case '1':
                    case '2':
                    case '3':

                        for (i = 0; i < (int)(move - '0'); i++)
                        {
                            x += deltaX;
                            y += deltaY;

                            if (x >= 20 || x < 0 || y >= 20 || y < 0)
                            {
                                goto gameOver;
                            }

                            switch (Policka[y, x])
                            {
                                case '#':
                                    goto gameOver;
                                case '$':
                                    break;
                                case 'X':
                                    x -= deltaX;
                                    y -= deltaY;
                                    goto endMove;
                            }
                        }
                        break;
                    case 'Z':
                        x -= deltaX;
                        y -= deltaY;

                        if (x >= 20 || x < 0 || y >= 20 || y < 0)
                        {
                            goto gameOver;
                        }

                        switch (Policka[y, x])
                        {
                            case '#':
                                goto gameOver;
                            case '$':
                                break;
                            case 'X':
                                x += deltaX;
                                y += deltaY;
                                break;
                        }
                        break;
                    case 'L':
                    case 'R':
                        switch (smer)
                        {
                            case 'N': smer = move == 'L' ? 'L' : 'R'; break;
                            case 'L': smer = move == 'L' ? 'D' : 'N'; break;
                            case 'D': smer = move == 'L' ? 'R' : 'L'; break;
                            case 'R': smer = move == 'L' ? 'N' : 'D'; break;
                        }

                        deltaX = 0;
                        deltaY = 0;
                        switch (smer)
                        {
                            case 'N': deltaY = -1; break;
                            case 'R': deltaX = 1; break;
                            case 'L': deltaX = -1; break;
                            case 'D': deltaY = 1; break;
                        }
                        break;
                }


            endMove:
                try
                {
                    switch (Policka[y, x])
                    {
                        case '^': y--; if (Policka[y, x] == 'X') y++; break;
                        case 'v': y++; if (Policka[y, x] == 'X') y--; break;
                        case '<': x--; if (Policka[y, x] == 'X') x++; break;
                        case '>': x++; if (Policka[y, x] == 'X') x--; break;
                    }
                }
                catch
                { }

                if (x >= 20 || x < 0 || y >= 20 || y < 0 || Policka[y, x] == '#')
                {
                    goto gameOver;
                }

                kroky.Add(new { X = x, Y = y, Smer = smer });

                if (Policka[y, x] == '$')
                {
                    goto win;
                }
            }

            win = false;
            gameOver = false;
            return kroky;

        win:
            win = true;
            gameOver = false;
            return kroky;

        gameOver:
            win = false;
            gameOver = true;
            return kroky;
        }

        public char[] FindMoves(int goalX, int goalY, int goalSmer)
        {
            return FindMoves(goalX, goalY, goalSmer, X, Y, Smer, Moznosti, 5);
        }

        // najde N pohybů, které dojdou na zvolené místo se zvoleným směrem
        public char[] FindMoves(int goalX, int goalY, int goalSmer, int x, int y, char smer, char[] vyber, int count)
        {
            bool win, gameOver;
            char[] newVyber = new char[vyber.Length - 1];

            for (int i = 0; i < vyber.Length; i++)
            {
                char c = vyber[i];
                List<dynamic> moves = CalculateMoves(new char[] { c }, x, y, smer, out win, out gameOver);
                if (gameOver)
                {
                    continue;
                }
                else if (win && Policka[goalY, goalX] == '$')
                {
                    return new char[] { c };
                }
                else if (count == 1)
                {
                    if (moves[0].X == goalX && moves[0].Y == goalY && moves[0].Smer == goalSmer)
                    {
                        return new char[] { c };
                    }
                    else
                    {
                        continue;
                    }
                }
                else
                {
                    if (i != 0) { Array.Copy(vyber, newVyber, i); }
                    Array.Copy(vyber, i + 1, newVyber, i, vyber.Length - i - 1);
                    char[] tmp = FindMoves(goalX, goalY, goalSmer, moves[0].X, moves[0].Y, moves[0].Smer, newVyber, count - 1);
                    if (tmp == null)
                    {
                        continue;
                    }
                    else
                    {
                        char[] result = new char[count];
                        result[0] = c;
                        Array.Copy(tmp, 0, result, 1, tmp.Length);
                        return result;
                    }
                }
            }

            return null;
        }
    }
}
