﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace RoboRace
{
    public partial class VisualBoard : UserControl
    {
        const int SquareSize = 25;

        public Board Board;
        public VisualField[,] Policka;
        public VisualField[] Moznosti;
        public int PocetVybranych;
        public VisualField PresunovanePole;
        public bool Win, GameOver;

        public VisualBoard()
        {
            InitializeComponent();
            Initialize();
            RefreshBoard();
        }

        private void Initialize()
        {
            Policka = new VisualField[20, 20];
            Moznosti = new VisualField[9];
            PocetVybranych = 0;

            int y, x, i;
            VisualField field;

            for (y = 0; y < 20; y++)
            {
                for (x = 0; x < 20; x++)
                {
                    field = new VisualField();
                    field.Width = SquareSize;
                    field.Height = SquareSize;
                    field.Left = ((SquareSize - 1) * x);
                    field.Top = ((SquareSize - 1) * y);
                    Policka[y, x] = field;
                    field.label1.MouseClick += new MouseEventHandler(label1_MouseClick);
                    this.Controls.Add(field);
                }
            }

            for (i = 0; i < 9; i++)
            {
                field = new VisualField();
                field.Width = SquareSize;
                field.Height = SquareSize;
                field.Left = ((SquareSize - 1) * i) + label1.Width + 20;
                field.Top = SquareSize * 23;
                field.label1.MouseClick += new MouseEventHandler(field_MouseClick);
                field.label1.MouseDoubleClick += new MouseEventHandler(label1_MouseDoubleClick);
                Moznosti[i] = field;
                this.Controls.Add(field);
            }
        }

        bool SearchInProgress = false;

        void label1_MouseClick(object sender, MouseEventArgs e)
        {
            lock (this)
            {
                if (SearchInProgress)
                {
                    return;
                }
                SearchInProgress = true;
            }

            Label label = (Label)sender;
            VisualField field = (VisualField)label.Parent;
            int x = field.Left / (SquareSize - 1);
            int y = field.Top / (SquareSize - 1);
            char smer = comboBox1.Text[0];
            char[] pohyby = Board.FindMoves(x, y, smer);
            if (pohyby == null)
            {
                MessageBox.Show("Nepodařilo se najít cestu.");
            }
            else
            {
                ResetMoznosti();
                for (int i = 0; i < pohyby.Length; i++)
                {
                    for (int j = i; j < 9; j++)
                    {
                        if (Board.Moznosti[j] == pohyby[i])
                        {
                            char tmp = Board.Moznosti[j];
                            Board.Moznosti[j] = Board.Moznosti[i];
                            Board.Moznosti[i] = tmp;
                        }
                    }
                }
                RefreshBoard();
                for (int i = 0; i < pohyby.Length; i++)
                {
                    Moznosti[i].IsActivated = true;
                }
                PocetVybranych = pohyby.Length;
                if (pohyby.Length < 5) Win = true;
                RefreshKroky();
            }
            SearchInProgress = false;
        }

        void field_MouseClick(object sender, MouseEventArgs e)
        {
            Label label = (Label)sender;
            VisualField field = (VisualField)label.Parent;
            int i;

            if (PresunovanePole == field)
            {
                return;
            }

            if (PresunovanePole == null)
            {
                if (field.IsActivated)
                {
                    field.IsActivated = false;
                    PocetVybranych--;
                }
                else
                {
                    if (PocetVybranych == 5)
                    {
                        return;
                    }
                    else
                    {
                        field.IsActivated = true;
                        PocetVybranych++;
                    }
                }
                RefreshKroky();
            }
            else
            {
                string tmp = field.label1.Text;
                field.SetText(PresunovanePole.label1.Text);
                PresunovanePole.SetText(tmp);
                PresunovanePole = null;

                for (i = 0; i < 9; i++)
                {
                    Board.Moznosti[i] = Moznosti[i].label1.Text[0];
                }
            }
        }

        void label1_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            Label label = (Label)sender;
            VisualField field = (VisualField)label.Parent;
            PresunovanePole = field;
            if (field.IsActivated)
            {
                field.IsActivated = false;
                PocetVybranych--;
            }
            else
            {
                if (PocetVybranych == 5)
                {
                    return;
                }
                else
                {
                    field.IsActivated = true;
                    PocetVybranych++;
                }
            }
            RefreshKroky();
        }

        // vykreslí kroky při provedení aktuálního výběru
        public void RefreshKroky()
        {
            RefreshBoard();
            List<char> kroky = new List<char>();
            int i;
            for (i = 0; i < 9; i++)
            {
                if (Moznosti[i].IsActivated)
                {
                    kroky.Add(Moznosti[i].label1.Text[0]);
                }
            }
            Board.Vyber = kroky.ToArray();

            List<dynamic> pozice = Board.CalculateMoves(out Win, out GameOver);
            for (i = 0; i < pozice.Count - 1; i++)
            {
                Policka[pozice[i].Y, pozice[i].X].CestaSmer = pozice[i].Smer;
                Policka[pozice[i].Y, pozice[i].X].Invalidate();
            }

            if (pozice.Count > 0)
            {
                Policka[pozice[pozice.Count - 1].Y, pozice[pozice.Count - 1].X].CestaSmer = pozice[i].Smer;
                Policka[pozice[pozice.Count - 1].Y, pozice[pozice.Count - 1].X].Invalidate();
            }
        }

        // resetuje výběr
        public void ResetMoznosti()
        {
            int i;
            for (i = 0; i < 9; i++)
            {
                Moznosti[i].IsActivated = false;
            }
            PocetVybranych = 0;
        }

        // překreslí hrací plochu
        public void RefreshBoard()
        {
            int y, x, i;

            for (y = 0; y < 20; y++)
            {
                for (x = 0; x < 20; x++)
                {
                    if (Board == null)
                    {
                        Policka[y, x].SetText("");
                        Policka[y, x].Smer = null;
                    }
                    else
                    {
                        Policka[y, x].SetText(Convert.ToString(Board.Policka[y, x]));
                        if (Board.X == x && Board.Y == y)
                        {
                            Policka[y, x].Smer = Board.Smer;
                        }
                        else
                        {
                            Policka[y, x].Smer = null;
                        }
                    }
                    Policka[y, x].CestaSmer = null;
                    Policka[y, x].Invalidate();
                }
            }

            for (i = 0; i < 9; i++)
            {
                Moznosti[i].SetText(Board == null ? "" : Convert.ToString(Board.Moznosti[i]));
            }

            button1.Text = Board == null ? "Nová hra" : "Konec hry";
            button2.Visible = Board != null;
            label3.Text = Board == null ? "" : "Zvolte 5 pohybů pomocí myši. Pohyb přesunete tak, že na něj dvakrát kliknete a poté jednou kliknete na cílovou pozici.";
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (Board == null)
            {
                Board = new Board();
                Board.BaseUri = "http://" + textBox1.Text + "/";
                Board.Start();
            }
            else
            {
                Board = null;
            }
            ResetMoznosti();
            RefreshBoard();
        }

        private void button2_Click(object sender, EventArgs e)
        {
            if (PocetVybranych == 5 || Win)
            {
                try
                {
                    Board.Execute();
                }
                catch (InvalidOperationException exception)
                {
                    MessageBox.Show(exception.Message);
                    Board = null;
                }
                ResetMoznosti();
                RefreshBoard();
            }
        }
    }
}
